@@ -39,18 +39,18 @@ const char *att_ecode2str(uint8_t status)
 	case ATT_ECODE_INVALID_HANDLE:
 		return "Invalid handle";
 	case ATT_ECODE_READ_NOT_PERM:
-		return "Atribute can't be read";
+		return "Attribute can't be read";
 	case ATT_ECODE_WRITE_NOT_PERM:
 		return "Attribute can't be written";
 	case ATT_ECODE_INVALID_PDU:
 		return "Attribute PDU was invalid";
-	case ATT_ECODE_INSUFF_AUTHEN:
+	case ATT_ECODE_AUTHENTICATION:
 		return "Attribute requires authentication before read/write";
 	case ATT_ECODE_REQ_NOT_SUPP:
 		return "Server doesn't support the request received";
 	case ATT_ECODE_INVALID_OFFSET:
 		return "Offset past the end of the attribute";
-	case ATT_ECODE_INSUFF_AUTHO:
+	case ATT_ECODE_AUTHORIZATION:
 		return "Attribute requires authorization before read/write";
 	case ATT_ECODE_PREP_QUEUE_FULL:
 		return "Too many prepare writes have been queued";
@@ -820,62 +820,65 @@ struct att_data_list *dec_find_info_resp(const uint8_t *pdu, int len,
 	return list;
 }
 
-uint16_t enc_notification(struct attribute *a, uint8_t *pdu, int len)
+uint16_t enc_notification(uint16_t handle, uint8_t *value, int vlen,
+						uint8_t *pdu, int len)
 {
 	const uint16_t min_len = sizeof(pdu[0]) + sizeof(uint16_t);
 
 	if (pdu == NULL)
 		return 0;
 
-	if (len < (a->len + min_len))
+	if (len < (vlen + min_len))
 		return 0;
 
 	pdu[0] = ATT_OP_HANDLE_NOTIFY;
-	att_put_u16(a->handle, &pdu[1]);
-	memcpy(&pdu[3], a->data, a->len);
+	att_put_u16(handle, &pdu[1]);
+	memcpy(&pdu[3], value, vlen);
 
-	return a->len + min_len;
+	return vlen + min_len;
 }
 
-uint16_t enc_indication(struct attribute *a, uint8_t *pdu, int len)
+uint16_t enc_indication(uint16_t handle, uint8_t *value, int vlen,
+						uint8_t *pdu, int len)
 {
 	const uint16_t min_len = sizeof(pdu[0]) + sizeof(uint16_t);
 
 	if (pdu == NULL)
 		return 0;
 
-	if (len < (a->len + min_len))
+	if (len < (vlen + min_len))
 		return 0;
 
 	pdu[0] = ATT_OP_HANDLE_IND;
-	att_put_u16(a->handle, &pdu[1]);
-	memcpy(&pdu[3], a->data, a->len);
+	att_put_u16(handle, &pdu[1]);
+	memcpy(&pdu[3], value, vlen);
 
-	return a->len + min_len;
+	return vlen + min_len;
 }
 
-struct attribute *dec_indication(const uint8_t *pdu, int len)
+uint16_t dec_indication(const uint8_t *pdu, int len, uint16_t *handle,
+						uint8_t *value, int vlen)
 {
 	const uint16_t min_len = sizeof(pdu[0]) + sizeof(uint16_t);
-
-	struct attribute *a;
+	uint16_t dlen;
 
 	if (pdu == NULL)
-		return NULL;
+		return 0;
 
 	if (pdu[0] != ATT_OP_HANDLE_IND)
-		return NULL;
+		return 0;
 
 	if (len < min_len)
-		return NULL;
+		return 0;
 
-	a = g_malloc0(sizeof(struct attribute) + len - min_len);
-	a->len = len - min_len;
+	dlen = MIN(len - min_len, vlen);
 
-	a->handle = att_get_u16(&pdu[1]);
-	memcpy(a->data, &pdu[3], a->len);
+	if (handle)
+		*handle = att_get_u16(&pdu[1]);
 
-	return a;
+	memcpy(value, &pdu[3], dlen);
+
+	return dlen;
 }
 
 uint16_t enc_confirmation(uint8_t *pdu, int len)
