@@ -34,6 +34,7 @@
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/uuid.h>
 
+#include "glib-compat.h"
 #include "adapter.h"
 #include "device.h"
 #include "log.h"
@@ -45,24 +46,12 @@
 
 #include "att.h"
 #include "gattrib.h"
+#include "attio.h"
 #include "gatt.h"
 #include "client.h"
 
 #define CHAR_INTERFACE "org.bluez.Characteristic"
 
-struct gatt_service {
-	struct btd_device *dev;
-	DBusConnection *conn;
-	bdaddr_t sba;
-	bdaddr_t dba;
-	char *path;
-	GSList *primary;
-	GAttrib *attrib;
-	DBusMessage *msg;
-	int psm;
-	gboolean listen;
-};
-
 struct format {
 	guint8 format;
 	guint8 exponent;
@@ -71,16 +60,28 @@ struct format {
 	guint16 desc;
 } __attribute__ ((packed));
 
-struct primary {
-	struct gatt_service *gatt;
-	struct att_primary *att;
+struct query {
+	DBusMessage *msg;
+	guint attioid;
+	GSList *list;
+};
+
+struct gatt_service {
+	struct btd_device *dev;
+	struct att_primary *prim;
+	DBusConnection *conn;
+	GAttrib *attrib;
+	guint attioid;
+	int psm;
 	char *path;
 	GSList *chars;
+	GSList *offline_chars;
 	GSList *watchers;
+	struct query *query;
 };
 
 struct characteristic {
-	struct primary *prim;
+	struct gatt_service *gatt;
 	char *path;
 	uint16_t handle;
 	uint16_t end;
@@ -94,9 +95,8 @@ struct characteristic {
 };
 
 struct query_data {
-	struct primary *prim;
+	struct gatt_service *gatt;
 	struct characteristic *chr;
-	DBusMessage *msg;
 	uint16_t handle;
 };
 
@@ -104,7 +104,7 @@ struct watcher {
 	guint id;
 	char *name;
 	char *path;
-	struct primary *prim;
+	struct gatt_service *gatt;
 };
 
 static GSList *gatt_services = NULL;
@@ -130,41 +130,26 @@ static void watcher_free(void *user_data)
 	g_free(watcher);
 }
 
-static void primary_free(void *user_data)
+static void gatt_service_free(struct gatt_service *gatt)
 {
-	struct primary *prim = user_data;
-	GSList *l;
-
-	for (l = prim->watchers; l; l = l->next) {
-		struct watcher *watcher = l->data;
-		g_dbus_remove_watch(prim->gatt->conn, watcher->id);
-	}
-
-	g_slist_foreach(prim->chars, (GFunc) characteristic_free, NULL);
-	g_slist_free(prim->chars);
-	g_free(prim->path);
-	g_free(prim);
-}
-
-static void gatt_service_free(void *user_data)
-{
-	struct gatt_service *gatt = user_data;
-
-	g_slist_foreach(gatt->primary, (GFunc) primary_free, NULL);
-	g_slist_free(gatt->primary);
-	g_attrib_unref(gatt->attrib);
+	g_slist_free_full(gatt->watchers, watcher_free);
+	g_slist_free_full(gatt->chars, characteristic_free);
+	g_slist_free(gatt->offline_chars);
 	g_free(gatt->path);
 	btd_device_unref(gatt->dev);
 	dbus_connection_unref(gatt->conn);
 	g_free(gatt);
 }
 
-static int gatt_dev_cmp(gconstpointer a, gconstpointer b)
+static void gatt_get_address(struct gatt_service *gatt,
+				bdaddr_t *sba, bdaddr_t *dba)
 {
-	const struct gatt_service *gatt = a;
-	const struct btd_device *dev = b;
+	struct btd_device *device = gatt->dev;
+	struct btd_adapter *adapter;
 
-	return gatt->dev != dev;
+	adapter = device_get_adapter(device);
+	adapter_get_address(adapter, sba);
+	device_get_address(device, dba, NULL);
 }
 
 static int characteristic_handle_cmp(gconstpointer a, gconstpointer b)
@@ -222,14 +207,11 @@ static void append_char_dict(DBusMessageIter *iter, struct characteristic *chr)
 static void watcher_exit(DBusConnection *conn, void *user_data)
 {
 	struct watcher *watcher = user_data;
-	struct primary *prim = watcher->prim;
-	struct gatt_service *gatt = prim->gatt;
+	struct gatt_service *gatt = watcher->gatt;
 
-	DBG("%s watcher %s exited", prim->path, watcher->name);
+	DBG("%s watcher %s exited", gatt->path, watcher->name);
 
-	prim->watchers = g_slist_remove(prim->watchers, watcher);
-
-	g_attrib_unref(gatt->attrib);
+	gatt->watchers = g_slist_remove(gatt->watchers, watcher);
 }
 
 static int characteristic_set_value(struct characteristic *chr,
@@ -249,7 +231,7 @@ static void update_watchers(gpointer data, gpointer user_data)
 {
 	struct watcher *w = data;
 	struct characteristic *chr = user_data;
-	DBusConnection *conn = w->prim->gatt->conn;
+	DBusConnection *conn = w->gatt->conn;
 	DBusMessage *msg;
 
 	msg = dbus_message_new_method_call(w->name, w->path,
@@ -270,8 +252,7 @@ static void events_handler(const uint8_t *pdu, uint16_t len,
 {
 	struct gatt_service *gatt = user_data;
 	struct characteristic *chr;
-	struct primary *prim;
-	GSList *lprim, *lchr;
+	GSList *l;
 	uint8_t opdu[ATT_MAX_MTU];
 	guint handle;
 	uint16_t olen;
@@ -284,17 +265,12 @@ static void events_handler(const uint8_t *pdu, uint16_t len,
 
 	handle = att_get_u16(&pdu[1]);
 
-	for (lprim = gatt->primary, prim = NULL, chr = NULL; lprim;
-						lprim = lprim->next) {
-		prim = lprim->data;
+	l = g_slist_find_custom(gatt->chars, GUINT_TO_POINTER(handle),
+						characteristic_handle_cmp);
+	if (!l)
+		return;
 
-		lchr = g_slist_find_custom(prim->chars,
-			GUINT_TO_POINTER(handle), characteristic_handle_cmp);
-		if (lchr) {
-			chr = lchr->data;
-			break;
-		}
-	}
+	chr = l->data;
 
 	if (chr == NULL) {
 		DBG("Attribute handle 0x%02x not found", handle);
@@ -310,131 +286,84 @@ static void events_handler(const uint8_t *pdu, uint16_t len,
 		if (characteristic_set_value(chr, &pdu[3], len - 3) < 0)
 			DBG("Can't change Characteristic 0x%02x", handle);
 
-		g_slist_foreach(prim->watchers, update_watchers, chr);
+		g_slist_foreach(gatt->watchers, update_watchers, chr);
 		break;
 	}
 }
 
-static void attrib_destroy(gpointer user_data)
+static void offline_char_written(gpointer user_data)
 {
-	struct gatt_service *gatt = user_data;
+	struct characteristic *chr = user_data;
+	struct gatt_service *gatt = chr->gatt;
 
-	gatt->attrib = NULL;
-}
+	gatt->offline_chars = g_slist_remove(gatt->offline_chars, chr);
 
-static void attrib_disconnect(gpointer user_data)
-{
-	struct gatt_service *gatt = user_data;
-
-	/* Remote initiated disconnection only */
-	g_attrib_unref(gatt->attrib);
-}
-
-static void connect_cb(GIOChannel *chan, GError *gerr, gpointer user_data)
-{
-	struct gatt_service *gatt = user_data;
-
-	if (gerr) {
-		if (gatt->msg) {
-			DBusMessage *reply = btd_error_failed(gatt->msg,
-							gerr->message);
-			g_dbus_send_message(gatt->conn, reply);
-		}
-
-		error("%s", gerr->message);
-		goto fail;
-	}
-
-	if (gatt->attrib == NULL)
+	if (gatt->offline_chars || gatt->watchers)
 		return;
 
-	/* Listen mode: used for notification and indication */
-	if (gatt->listen == TRUE) {
-		g_attrib_register(gatt->attrib,
-					ATT_OP_HANDLE_NOTIFY,
-					events_handler, gatt, NULL);
-		g_attrib_register(gatt->attrib,
-					ATT_OP_HANDLE_IND,
-					events_handler, gatt, NULL);
-		return;
-	}
-
-	return;
-fail:
-	g_attrib_unref(gatt->attrib);
+	btd_device_remove_attio_callback(gatt->dev, gatt->attioid);
+	gatt->attioid = 0;
 }
 
-static int l2cap_connect(struct gatt_service *gatt, GError **gerr,
-								gboolean listen)
+static void offline_char_write(gpointer data, gpointer user_data)
 {
-	GIOChannel *io;
+	struct characteristic *chr = data;
+	GAttrib *attrib = user_data;
 
-	if (gatt->attrib != NULL) {
-		gatt->attrib = g_attrib_ref(gatt->attrib);
-		gatt->listen = listen;
-		return 0;
+	gatt_write_cmd(attrib, chr->handle, chr->value, chr->vlen,
+						offline_char_written, chr);
+}
+
+static void attio_connected(GAttrib *attrib, gpointer user_data)
+{
+	struct gatt_service *gatt = user_data;
+
+	gatt->attrib = g_attrib_ref(attrib);
+
+	g_attrib_register(gatt->attrib, ATT_OP_HANDLE_NOTIFY,
+					events_handler, gatt, NULL);
+	g_attrib_register(gatt->attrib, ATT_OP_HANDLE_IND,
+					events_handler, gatt, NULL);
+
+	g_slist_foreach(gatt->offline_chars, offline_char_write, attrib);
+}
+
+static void attio_disconnected(gpointer user_data)
+{
+	struct gatt_service *gatt = user_data;
+
+	if (gatt->attrib) {
+		g_attrib_unref(gatt->attrib);
+		gatt->attrib = NULL;
 	}
-
-	/*
-	 * FIXME: If the service doesn't support Client Characteristic
-	 * Configuration it is necessary to poll the server from time
-	 * to time checking for modifications.
-	 */
-	if (gatt->psm < 0)
-		io = bt_io_connect(BT_IO_L2CAP, connect_cb, gatt, NULL, gerr,
-			BT_IO_OPT_SOURCE_BDADDR, &gatt->sba,
-			BT_IO_OPT_DEST_BDADDR, &gatt->dba,
-			BT_IO_OPT_CID, ATT_CID,
-			BT_IO_OPT_SEC_LEVEL, BT_IO_SEC_LOW,
-			BT_IO_OPT_INVALID);
-	else
-		io = bt_io_connect(BT_IO_L2CAP, connect_cb, gatt, NULL, gerr,
-			BT_IO_OPT_SOURCE_BDADDR, &gatt->sba,
-			BT_IO_OPT_DEST_BDADDR, &gatt->dba,
-			BT_IO_OPT_PSM, gatt->psm,
-			BT_IO_OPT_SEC_LEVEL, BT_IO_SEC_LOW,
-			BT_IO_OPT_INVALID);
-	if (!io)
-		return -1;
-
-	gatt->attrib = g_attrib_new(io);
-	g_io_channel_unref(io);
-	gatt->listen = listen;
-
-	g_attrib_set_destroy_function(gatt->attrib, attrib_destroy, gatt);
-	g_attrib_set_disconnect_function(gatt->attrib, attrib_disconnect,
-									gatt);
-
-	return 0;
 }
 
 static DBusMessage *register_watcher(DBusConnection *conn,
 						DBusMessage *msg, void *data)
 {
 	const char *sender = dbus_message_get_sender(msg);
-	struct primary *prim = data;
+	struct gatt_service *gatt = data;
 	struct watcher *watcher;
-	GError *gerr = NULL;
 	char *path;
 
 	if (!dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
 							DBUS_TYPE_INVALID))
 		return btd_error_invalid_args(msg);
 
-	if (l2cap_connect(prim->gatt, &gerr, TRUE) < 0) {
-		DBusMessage *reply = btd_error_failed(msg, gerr->message);
-		g_error_free(gerr);
-		return reply;
-	}
-
 	watcher = g_new0(struct watcher, 1);
 	watcher->name = g_strdup(sender);
-	watcher->prim = prim;
+	watcher->gatt = gatt;
 	watcher->path = g_strdup(path);
 	watcher->id = g_dbus_add_disconnect_watch(conn, sender, watcher_exit,
 							watcher, watcher_free);
 
-	prim->watchers = g_slist_append(prim->watchers, watcher);
+	if (gatt->attioid == 0)
+		gatt->attioid = btd_device_add_attio_callback(gatt->dev,
+							attio_connected,
+							attio_disconnected,
+							gatt);
+
+	gatt->watchers = g_slist_append(gatt->watchers, watcher);
 
 	return dbus_message_new_method_return(msg);
 }
@@ -443,7 +372,7 @@ static DBusMessage *unregister_watcher(DBusConnection *conn,
 						DBusMessage *msg, void *data)
 {
 	const char *sender = dbus_message_get_sender(msg);
-	struct primary *prim = data;
+	struct gatt_service *gatt = data;
 	struct watcher *watcher, *match;
 	GSList *l;
 	char *path;
@@ -455,25 +384,29 @@ static DBusMessage *unregister_watcher(DBusConnection *conn,
 	match = g_new0(struct watcher, 1);
 	match->name = g_strdup(sender);
 	match->path = g_strdup(path);
-	l = g_slist_find_custom(prim->watchers, match, watcher_cmp);
+	l = g_slist_find_custom(gatt->watchers, match, watcher_cmp);
 	watcher_free(match);
 	if (!l)
 		return btd_error_not_authorized(msg);
 
 	watcher = l->data;
 	g_dbus_remove_watch(conn, watcher->id);
-	prim->watchers = g_slist_remove(prim->watchers, watcher);
+	gatt->watchers = g_slist_remove(gatt->watchers, watcher);
 	watcher_free(watcher);
 
+	if (gatt->watchers == NULL && gatt->attioid) {
+		btd_device_remove_attio_callback(gatt->dev, gatt->attioid);
+		gatt->attioid = 0;
+	}
+
 	return dbus_message_new_method_return(msg);
 }
 
 static DBusMessage *set_value(DBusConnection *conn, DBusMessage *msg,
 			DBusMessageIter *iter, struct characteristic *chr)
 {
-	struct gatt_service *gatt = chr->prim->gatt;
+	struct gatt_service *gatt = chr->gatt;
 	DBusMessageIter sub;
-	GError *gerr = NULL;
 	uint8_t *value;
 	int len;
 
@@ -485,16 +418,20 @@ static DBusMessage *set_value(DBusConnection *conn, DBusMessage *msg,
 
 	dbus_message_iter_get_fixed_array(&sub, &value, &len);
 
-	if (l2cap_connect(gatt, &gerr, FALSE) < 0) {
-		DBusMessage *reply = btd_error_failed(msg, gerr->message);
-		g_error_free(gerr);
-		return reply;
-	}
-
-	gatt_write_cmd(gatt->attrib, chr->handle, value, len, NULL, NULL);
-
 	characteristic_set_value(chr, value, len);
 
+	if (gatt->attioid == 0)
+		gatt->attioid = btd_device_add_attio_callback(gatt->dev,
+							attio_connected,
+							attio_disconnected,
+							gatt);
+
+	if (gatt->attrib)
+		gatt_write_cmd(gatt->attrib, chr->handle, value, len,
+								NULL, NULL);
+	else
+		gatt->offline_chars = g_slist_append(gatt->offline_chars, chr);
+
 	return dbus_message_new_method_return(msg);
 }
 
@@ -573,34 +510,34 @@ static char *characteristic_list_to_string(GSList *chars)
 	return g_string_free(characteristics, FALSE);
 }
 
-static void store_characteristics(struct gatt_service *gatt,
-							struct primary *prim)
+static void store_characteristics(const bdaddr_t *sba, const bdaddr_t *dba,
+						uint16_t start, GSList *chars)
 {
 	char *characteristics;
-	struct att_primary *att = prim->att;
 
-	characteristics = characteristic_list_to_string(prim->chars);
+	characteristics = characteristic_list_to_string(chars);
 
-	write_device_characteristics(&gatt->sba, &gatt->dba, att->start,
-							characteristics);
+	write_device_characteristics(sba, dba, start, characteristics);
 
 	g_free(characteristics);
 }
 
-static void register_characteristics(struct primary *prim)
+static void register_characteristic(gpointer data, gpointer user_data)
 {
-	GSList *lc;
+	struct characteristic *chr = data;
+	DBusConnection *conn = chr->gatt->conn;
+	const char *gatt_path = user_data;
 
-	for (lc = prim->chars; lc; lc = lc->next) {
-		struct characteristic *chr = lc->data;
-		g_dbus_register_interface(prim->gatt->conn, chr->path,
-				CHAR_INTERFACE, char_methods,
-				NULL, NULL, chr, NULL);
-		DBG("Registered: %s", chr->path);
-	}
+	chr->path = g_strdup_printf("%s/characteristic%04x", gatt_path,
+								chr->handle);
+
+	g_dbus_register_interface(conn, chr->path, CHAR_INTERFACE,
+					char_methods, NULL, NULL, chr, NULL);
+
+	DBG("Registered: %s", chr->path);
 }
 
-static GSList *string_to_characteristic_list(struct primary *prim,
+static GSList *string_to_characteristic_list(struct gatt_service *gatt,
 							const char *str)
 {
 	GSList *l = NULL;
@@ -627,10 +564,7 @@ static GSList *string_to_characteristic_list(struct primary *prim,
 			continue;
 		}
 
-		chr->prim = prim;
-		chr->path = g_strdup_printf("%s/characteristic%04x",
-						prim->path, chr->handle);
-
+		chr->gatt = gatt;
 		l = g_slist_append(l, chr);
 	}
 
@@ -639,39 +573,29 @@ static GSList *string_to_characteristic_list(struct primary *prim,
 	return l;
 }
 
-static void load_characteristics(gpointer data, gpointer user_data)
+static GSList *load_characteristics(struct gatt_service *gatt, uint16_t start)
 {
-	struct primary *prim = data;
-	struct att_primary *att = prim->att;
-	struct gatt_service *gatt = user_data;
 	GSList *chrs_list;
+	bdaddr_t sba, dba;
 	char *str;
 
-	if (prim->chars) {
-		DBG("Characteristics already loaded");
-		return;
-	}
+	gatt_get_address(gatt, &sba, &dba);
 
-	str = read_device_characteristics(&gatt->sba, &gatt->dba, att->start);
+	str = read_device_characteristics(&sba, &dba, start);
 	if (str == NULL)
-		return;
+		return NULL;
 
-	chrs_list = string_to_characteristic_list(prim, str);
+	chrs_list = string_to_characteristic_list(gatt, str);
 
 	free(str);
 
-	if (chrs_list == NULL)
-		return;
-
-	prim->chars = chrs_list;
-	register_characteristics(prim);
-
-	return;
+	return chrs_list;
 }
 
 static void store_attribute(struct gatt_service *gatt, uint16_t handle,
 				uint16_t type, uint8_t *value, gsize len)
 {
+	bdaddr_t sba, dba;
 	bt_uuid_t uuid;
 	char *str, *tmp;
 	guint i;
@@ -686,15 +610,39 @@ static void store_attribute(struct gatt_service *gatt, uint16_t handle,
 	for (i = 0, tmp = str + MAX_LEN_UUID_STR; i < len; i++, tmp += 2)
 		sprintf(tmp, "%02X", value[i]);
 
-	write_device_attribute(&gatt->sba, &gatt->dba, handle, str);
+	gatt_get_address(gatt, &sba, &dba);
+
+	write_device_attribute(&sba, &dba, handle, str);
+
 	g_free(str);
 }
 
+static void query_list_append(struct gatt_service *gatt, struct query_data *data)
+{
+	struct query *query = gatt->query;
+
+	query->list = g_slist_append(query->list, data);
+}
+
+static void query_list_remove(struct gatt_service *gatt, struct query_data *data)
+{
+	struct query *query = gatt->query;
+
+	query->list = g_slist_remove(query->list, data);
+	if (query->list != NULL)
+		return;
+
+	btd_device_remove_attio_callback(gatt->dev, query->attioid);
+	g_free(query);
+
+	gatt->query = NULL;
+}
+
 static void update_char_desc(guint8 status, const guint8 *pdu, guint16 len,
 							gpointer user_data)
 {
 	struct query_data *current = user_data;
-	struct gatt_service *gatt = current->prim->gatt;
+	struct gatt_service *gatt = current->gatt;
 	struct characteristic *chr = current->chr;
 
 	if (status == 0) {
@@ -720,7 +668,7 @@ static void update_char_desc(guint8 status, const guint8 *pdu, guint16 len,
 		}
 	}
 
-	g_attrib_unref(gatt->attrib);
+	query_list_remove(gatt, current);
 	g_free(current);
 }
 
@@ -728,7 +676,7 @@ static void update_char_format(guint8 status, const guint8 *pdu, guint16 len,
 								gpointer user_data)
 {
 	struct query_data *current = user_data;
-	struct gatt_service *gatt = current->prim->gatt;
+	struct gatt_service *gatt = current->gatt;
 	struct characteristic *chr = current->chr;
 
 	if (status != 0)
@@ -746,7 +694,7 @@ static void update_char_format(guint8 status, const guint8 *pdu, guint16 len,
 				(void *) chr->format, sizeof(*chr->format));
 
 done:
-	g_attrib_unref(gatt->attrib);
+	query_list_remove(gatt, current);
 	g_free(current);
 }
 
@@ -754,7 +702,7 @@ static void update_char_value(guint8 status, const guint8 *pdu,
 					guint16 len, gpointer user_data)
 {
 	struct query_data *current = user_data;
-	struct gatt_service *gatt = current->prim->gatt;
+	struct gatt_service *gatt = current->gatt;
 	struct characteristic *chr = current->chr;
 
 	if (status == 0)
@@ -771,7 +719,7 @@ static void update_char_value(guint8 status, const guint8 *pdu,
 		}
 	}
 
-	g_attrib_unref(gatt->attrib);
+	query_list_remove(gatt, current);
 	g_free(current);
 }
 
@@ -788,7 +736,7 @@ static void descriptor_cb(guint8 status, const guint8 *pdu, guint16 plen,
 							gpointer user_data)
 {
 	struct query_data *current = user_data;
-	struct gatt_service *gatt = current->prim->gatt;
+	struct gatt_service *gatt = current->gatt;
 	struct att_data_list *list;
 	guint8 format;
 	int i;
@@ -820,16 +768,16 @@ static void descriptor_cb(guint8 status, const guint8 *pdu, guint16 plen,
 			continue;
 		}
 		qfmt = g_new0(struct query_data, 1);
-		qfmt->prim = current->prim;
+		qfmt->gatt = current->gatt;
 		qfmt->chr = current->chr;
 		qfmt->handle = handle;
 
 		if (uuid_desc16_cmp(&uuid, GATT_CHARAC_USER_DESC_UUID) == 0) {
-			gatt->attrib = g_attrib_ref(gatt->attrib);
+			query_list_append(gatt, qfmt);
 			gatt_read_char(gatt->attrib, handle, 0, update_char_desc,
 									qfmt);
 		} else if (uuid_desc16_cmp(&uuid, GATT_CHARAC_FMT_UUID) == 0) {
-			gatt->attrib = g_attrib_ref(gatt->attrib);
+			query_list_append(gatt, qfmt);
 			gatt_read_char(gatt->attrib, handle, 0,
 						update_char_format, qfmt);
 		} else
@@ -838,7 +786,7 @@ static void descriptor_cb(guint8 status, const guint8 *pdu, guint16 plen,
 
 	att_data_list_free(list);
 done:
-	g_attrib_unref(gatt->attrib);
+	query_list_remove(gatt, current);
 	g_free(current);
 }
 
@@ -846,22 +794,23 @@ static void update_all_chars(gpointer data, gpointer user_data)
 {
 	struct query_data *qdesc, *qvalue;
 	struct characteristic *chr = data;
-	struct primary *prim = user_data;
-	struct gatt_service *gatt = prim->gatt;
+	struct gatt_service *gatt = user_data;
 
 	qdesc = g_new0(struct query_data, 1);
-	qdesc->prim = prim;
+	qdesc->gatt = gatt;
 	qdesc->chr = chr;
 
-	gatt->attrib = g_attrib_ref(gatt->attrib);
+	query_list_append(gatt, qdesc);
+
 	gatt_find_info(gatt->attrib, chr->handle + 1, chr->end, descriptor_cb,
 									qdesc);
 
 	qvalue = g_new0(struct query_data, 1);
-	qvalue->prim = prim;
+	qvalue->gatt = gatt;
 	qvalue->chr = chr;
 
-	gatt->attrib = g_attrib_ref(gatt->attrib);
+	query_list_append(gatt, qvalue);
+
 	gatt_read_char(gatt->attrib, chr->handle, 0, update_char_value, qvalue);
 }
 
@@ -871,17 +820,17 @@ static void char_discovered_cb(GSList *characteristics, guint8 status,
 	DBusMessage *reply;
 	DBusMessageIter iter, array_iter;
 	struct query_data *current = user_data;
-	struct primary *prim = current->prim;
-	struct att_primary *att = prim->att;
-	struct gatt_service *gatt = prim->gatt;
+	struct gatt_service *gatt = current->gatt;
+	struct att_primary *prim = gatt->prim;
 	uint16_t *previous_end = NULL;
 	GSList *l;
+	bdaddr_t sba, dba;
 
 	if (status != 0) {
 		const char *str = att_ecode2str(status);
 
 		DBG("Discover all characteristics failed: %s", str);
-		reply = btd_error_failed(current->msg, str);
+		reply = btd_error_failed(gatt->query->msg, str);
 		goto fail;
 	}
 
@@ -891,17 +840,15 @@ static void char_discovered_cb(GSList *characteristics, guint8 status,
 		guint handle = current_chr->value_handle;
 		GSList *lchr;
 
-		lchr = g_slist_find_custom(prim->chars,
+		lchr = g_slist_find_custom(gatt->chars,
 			GUINT_TO_POINTER(handle), characteristic_handle_cmp);
 		if (lchr)
 			continue;
 
 		chr = g_new0(struct characteristic, 1);
-		chr->prim = prim;
+		chr->gatt = gatt;
 		chr->perm = current_chr->properties;
 		chr->handle = current_chr->value_handle;
-		chr->path = g_strdup_printf("%s/characteristic%04x",
-						prim->path, chr->handle);
 		strncpy(chr->type, current_chr->uuid, sizeof(chr->type));
 
 		if (previous_end)
@@ -909,23 +856,25 @@ static void char_discovered_cb(GSList *characteristics, guint8 status,
 
 		previous_end = &chr->end;
 
-		prim->chars = g_slist_append(prim->chars, chr);
+		gatt->chars = g_slist_append(gatt->chars, chr);
 	}
 
 	if (previous_end)
-		*previous_end = att->end;
+		*previous_end = prim->end;
 
-	store_characteristics(gatt, prim);
-	register_characteristics(prim);
+	gatt_get_address(gatt, &sba, &dba);
+	store_characteristics(&sba, &dba, prim->start, gatt->chars);
 
-	reply = dbus_message_new_method_return(current->msg);
+	g_slist_foreach(gatt->chars, register_characteristic, gatt->path);
+
+	reply = dbus_message_new_method_return(gatt->query->msg);
 
 	dbus_message_iter_init_append(reply, &iter);
 
 	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
 				DBUS_TYPE_OBJECT_PATH_AS_STRING, &array_iter);
 
-	for (l = prim->chars; l; l = l->next) {
+	for (l = gatt->chars; l; l = l->next) {
 		struct characteristic *chr = l->data;
 
 		dbus_message_iter_append_basic(&array_iter,
@@ -934,35 +883,59 @@ static void char_discovered_cb(GSList *characteristics, guint8 status,
 
 	dbus_message_iter_close_container(&iter, &array_iter);
 
-	g_slist_foreach(prim->chars, update_all_chars, prim);
+	g_slist_foreach(gatt->chars, update_all_chars, gatt);
 
 fail:
 	g_dbus_send_message(gatt->conn, reply);
-	g_attrib_unref(gatt->attrib);
+	query_list_remove(gatt, current);
 	g_free(current);
 }
 
+static void send_discover(GAttrib *attrib, gpointer user_data)
+{
+	struct query_data *qchr = user_data;
+	struct gatt_service *gatt = qchr->gatt;
+	struct att_primary *prim = gatt->prim;
+
+	gatt->attrib = g_attrib_ref(attrib);
+
+	gatt_discover_char(gatt->attrib, prim->start, prim->end, NULL,
+						char_discovered_cb, qchr);
+}
+
+static void cancel_discover(gpointer user_data)
+{
+	struct query_data *qchr = user_data;
+	struct gatt_service *gatt = qchr->gatt;
+
+	g_attrib_unref(gatt->attrib);
+	gatt->attrib = NULL;
+}
+
 static DBusMessage *discover_char(DBusConnection *conn, DBusMessage *msg,
 								void *data)
 {
-	struct primary *prim = data;
-	struct att_primary *att = prim->att;
-	struct gatt_service *gatt = prim->gatt;
+	struct gatt_service *gatt = data;
+	struct query *query;
 	struct query_data *qchr;
-	GError *gerr = NULL;
 
-	if (l2cap_connect(prim->gatt, &gerr, FALSE) < 0) {
-		DBusMessage *reply = btd_error_failed(msg, gerr->message);
-		g_error_free(gerr);
-		return reply;
-	}
+	if (gatt->query)
+		return btd_error_busy(msg);
+
+	query = g_new0(struct query, 1);
 
 	qchr = g_new0(struct query_data, 1);
-	qchr->prim = prim;
-	qchr->msg = dbus_message_ref(msg);
+	qchr->gatt = gatt;
 
-	gatt_discover_char(gatt->attrib, att->start, att->end, NULL,
-						char_discovered_cb, qchr);
+	query->msg = dbus_message_ref(msg);
+	query->attioid = btd_device_add_attio_callback(gatt->dev,
+							send_discover,
+							cancel_discover,
+							qchr);
+
+	gatt->query = query;
+
+	query_list_append(gatt, qchr);
 
 	return NULL;
 }
@@ -970,7 +943,7 @@ static DBusMessage *discover_char(DBusConnection *conn, DBusMessage *msg,
 static DBusMessage *prim_get_properties(DBusConnection *conn, DBusMessage *msg,
 								void *data)
 {
-	struct primary *prim = data;
+	struct gatt_service *gatt = data;
 	DBusMessage *reply;
 	DBusMessageIter iter;
 	DBusMessageIter dict;
@@ -990,16 +963,16 @@ static DBusMessage *prim_get_properties(DBusConnection *conn, DBusMessage *msg,
 			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
 			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
 
-	chars = g_new0(char *, g_slist_length(prim->chars) + 1);
+	chars = g_new0(char *, g_slist_length(gatt->chars) + 1);
 
-	for (i = 0, l = prim->chars; l; l = l->next, i++) {
+	for (i = 0, l = gatt->chars; l; l = l->next, i++) {
 		struct characteristic *chr = l->data;
 		chars[i] = chr->path;
 	}
 
 	dict_append_array(&dict, "Characteristics", DBUS_TYPE_OBJECT_PATH,
 								&chars, i);
-	uuid = prim->att->uuid;
+	uuid = gatt->prim->uuid;
 	dict_append_entry(&dict, "UUID", DBUS_TYPE_STRING, &uuid);
 
 	g_free(chars);
@@ -1020,84 +993,95 @@ static GDBusMethodTable prim_methods[] = {
 	{ }
 };
 
-static GSList *register_primaries(struct gatt_service *gatt, GSList *primaries)
+static struct gatt_service *primary_register(DBusConnection *conn,
+						struct btd_device *device,
+						struct att_primary *prim,
+						int psm)
 {
-	GSList *l, *paths;
+	struct gatt_service *gatt;
+	const char *device_path;
 
-	for (paths = NULL, l = primaries; l; l = l->next) {
-		struct att_primary *att = l->data;
-		struct primary *prim;
+	device_path = device_get_path(device);
 
-		prim = g_new0(struct primary, 1);
-		prim->att = att;
-		prim->gatt = gatt;
-		prim->path = g_strdup_printf("%s/service%04x", gatt->path,
-								att->start);
+	gatt = g_new0(struct gatt_service, 1);
+	gatt->dev = btd_device_ref(device);
+	gatt->prim = prim;
+	gatt->psm = psm;
+	gatt->conn = dbus_connection_ref(conn);
+	gatt->path = g_strdup_printf("%s/service%04x", device_path,
+								prim->start);
 
-		g_dbus_register_interface(gatt->conn, prim->path,
-				CHAR_INTERFACE, prim_methods,
-				NULL, NULL, prim, NULL);
-		DBG("Registered: %s", prim->path);
+	g_dbus_register_interface(gatt->conn, gatt->path,
+					CHAR_INTERFACE, prim_methods,
+					NULL, NULL, gatt, NULL);
+	gatt->chars = load_characteristics(gatt, prim->start);
+	g_slist_foreach(gatt->chars, register_characteristic, gatt->path);
 
-		gatt->primary = g_slist_append(gatt->primary, prim);
-		paths = g_slist_append(paths, g_strdup(prim->path));
-		load_characteristics(prim, gatt);
-	}
-
-	return paths;
+	return gatt;
 }
 
 GSList *attrib_client_register(DBusConnection *connection,
 					struct btd_device *device, int psm,
 					GAttrib *attrib, GSList *primaries)
 {
-	struct btd_adapter *adapter = device_get_adapter(device);
-	const char *path = device_get_path(device);
-	struct gatt_service *gatt;
-	bdaddr_t sba, dba;
+	GSList *l, *services;
 
-	adapter_get_address(adapter, &sba);
-	device_get_address(device, &dba);
+	for (l = primaries, services = NULL; l; l = l->next) {
+		struct att_primary *prim = l->data;
+		struct gatt_service *gatt;
 
-	gatt = g_new0(struct gatt_service, 1);
-	gatt->dev = btd_device_ref(device);
-	gatt->conn = dbus_connection_ref(connection);
-	gatt->listen = FALSE;
-	gatt->path = g_strdup(path);
-	bacpy(&gatt->sba, &sba);
-	bacpy(&gatt->dba, &dba);
-	gatt->psm = psm;
+		gatt = primary_register(connection, device, prim, psm);
 
-	if (attrib)
-		gatt->attrib = g_attrib_ref(attrib);
+		DBG("Registered: %s", gatt->path);
 
-	gatt_services = g_slist_append(gatt_services, gatt);
+		services = g_slist_append(services, g_strdup(gatt->path));
+		gatt_services = g_slist_append(gatt_services, gatt);
 
-	return register_primaries(gatt, primaries);
-}
-
-void attrib_client_unregister(struct btd_device *device)
-{
-	struct gatt_service *gatt;
-	GSList *l, *lp, *lc;
-
-	l = g_slist_find_custom(gatt_services, device, gatt_dev_cmp);
-	if (!l)
-		return;
-
-	gatt = l->data;
-	gatt_services = g_slist_remove(gatt_services, gatt);
-
-	for (lp = gatt->primary; lp; lp = lp->next) {
-		struct primary *prim = lp->data;
-		for (lc = prim->chars; lc; lc = lc->next) {
-			struct characteristic *chr = lc->data;
-			g_dbus_unregister_interface(gatt->conn, chr->path,
-								CHAR_INTERFACE);
-		}
-		g_dbus_unregister_interface(gatt->conn, prim->path,
-								CHAR_INTERFACE);
 	}
 
-	gatt_service_free(gatt);
+	return services;
+}
+
+static void primary_unregister(struct gatt_service *gatt)
+{
+	GSList *l;
+
+	if (gatt->attioid)
+		btd_device_remove_attio_callback(gatt->dev, gatt->attioid);
+
+	for (l = gatt->chars; l; l = l->next) {
+		struct characteristic *chr = l->data;
+		g_dbus_unregister_interface(gatt->conn, chr->path,
+							CHAR_INTERFACE);
+	}
+
+	g_dbus_unregister_interface(gatt->conn, gatt->path, CHAR_INTERFACE);
+}
+
+static int path_cmp(gconstpointer data, gconstpointer user_data)
+{
+	const char *path = data;
+	const char *gatt_path = user_data;
+
+	return g_strcmp0(path, gatt_path);
+}
+
+void attrib_client_unregister(GSList *services)
+{
+	GSList *l, *left;
+
+	for (l = gatt_services, left = NULL; l; l = l->next) {
+		struct gatt_service *gatt = l->data;
+
+		if (!g_slist_find_custom(services, gatt->path, path_cmp)) {
+			left = g_slist_append(left, gatt);
+			continue;
+		}
+
+		primary_unregister(gatt);
+		gatt_service_free(gatt);
+	}
+
+	g_slist_free(gatt_services);
+	gatt_services = left;
 }
