@@ -42,7 +42,6 @@
 #include <gdbus.h>
 
 #include "log.h"
-#include "textfile.h"
 #include "../src/adapter.h"
 #include "../src/device.h"
 
@@ -53,6 +52,8 @@
 #include "unix.h"
 #include "avdtp.h"
 #include "control.h"
+#include "avctp.h"
+#include "avrcp.h"
 #include "headset.h"
 #include "gateway.h"
 #include "sink.h"
@@ -62,6 +63,7 @@
 
 #define CONTROL_CONNECT_TIMEOUT 2
 #define AVDTP_CONNECT_TIMEOUT 1
+#define AVDTP_CONNECT_TIMEOUT_BOOST 1
 #define HEADSET_CONNECT_TIMEOUT 1
 
 typedef enum {
@@ -306,6 +308,7 @@ static gboolean avdtp_connect_timeout(gpointer user_data)
 static gboolean device_set_avdtp_timer(struct audio_device *dev)
 {
 	struct dev_priv *priv = dev->priv;
+	guint timeout = AVDTP_CONNECT_TIMEOUT;
 
 	if (!dev->sink)
 		return FALSE;
@@ -313,7 +316,12 @@ static gboolean device_set_avdtp_timer(struct audio_device *dev)
 	if (priv->avdtp_timer)
 		return FALSE;
 
-	priv->avdtp_timer = g_timeout_add_seconds(AVDTP_CONNECT_TIMEOUT,
+	/* If the headset is the HSP/HFP RFCOMM initiator, give the headset
+	 * time to initiate AVDTP signalling (and avoid further racing) */
+	if (dev->headset && headset_get_rfcomm_initiator(dev))
+		timeout += AVDTP_CONNECT_TIMEOUT_BOOST;
+
+	priv->avdtp_timer = g_timeout_add_seconds(timeout,
 							avdtp_connect_timeout,
 							dev);
 
