@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2010  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This library is free software; you can redistribute it and/or
@@ -91,7 +91,7 @@ const gchar *gst_sbc_get_allocation_from_list(const GValue *value)
  */
 const gchar *gst_sbc_get_mode_from_list(const GValue *list, gint channels)
 {
-	int i;
+	unsigned int i;
 	const GValue *value;
 	const gchar *aux;
 	gboolean joint, stereo, dual, mono;
@@ -311,7 +311,7 @@ GstCaps *gst_sbc_parse_caps_from_sbc(sbc_t *sbc)
 }
 
 /*
- * Given a GstCaps, this will return a fixed GstCaps on sucessfull conversion.
+ * Given a GstCaps, this will return a fixed GstCaps on successful conversion.
  * If an error occurs, it will return NULL and error_message will contain the
  * error message.
  *
