@@ -5,11 +5,6 @@
  *  Copyright (C) 2010 GSyC/LibreSoft, Universidad Rey Juan Carlos.
  *  Copyright (C) 2010 Signove
  *
- *  Authors:
- *  Santiago Carot-Nemesio <sancane at gmail.com>
- *  Jose Antonio Santos-Cadenas <santoscadenas at gmail.com>
- *  Elvis Pfützenreuter <epx at signove.com>
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -26,21 +21,23 @@
  *
  */
 
-#include "btio.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdint.h>
 #include <netinet/in.h>
 #include <time.h>
 #include <stdlib.h>
-#include <bluetooth/bluetooth.h>
-#include <bluetooth/l2cap.h>
-#include "../src/adapter.h"
-#include "../src/manager.h"
 #include <sys/ioctl.h>
 
-#include "config.h"
-#include "log.h"
-
 #include <bluetooth/bluetooth.h>
+#include <bluetooth/l2cap.h>
+#include <adapter.h>
+#include <manager.h>
+#include <btio.h>
+#include <log.h>
+
 #include "mcap.h"
 #include "mcap_lib.h"
 #include "mcap_internal.h"
@@ -92,21 +89,6 @@ struct sync_set_data {
 	gboolean role;
 };
 
-/* Ripped from lib/sdp.c */
-
-#if __BYTE_ORDER == __BIG_ENDIAN
-#define ntoh64(x) (x)
-#else
-static inline uint64_t ntoh64(uint64_t n)
-{
-        uint64_t h;
-        uint64_t tmp = ntohl(n & 0x00000000ffffffff);
-        h = ntohl(n >> 32);
-        h |= tmp << 32;
-        return h;
-}
-#endif
-
 #define hton64(x)     ntoh64(x)
 
 static gboolean csp_caps_initialized = FALSE;
@@ -246,7 +228,7 @@ static gboolean valid_btclock(uint32_t btclk)
 static gboolean read_btclock(struct mcap_mcl *mcl, uint32_t *btclock,
 							uint16_t *btaccuracy)
 {
-	int ret, handle, which = 1;
+	int which = 1;
 	struct btd_adapter *adapter;
 
 	adapter = manager_find_adapter(&mcl->mi->src);
@@ -254,13 +236,11 @@ static gboolean read_btclock(struct mcap_mcl *mcl, uint32_t *btclock,
 	if (!adapter)
 		return FALSE;
 
-	if (btd_adapter_get_conn_handle(adapter, &mcl->addr, &handle))
+	if (btd_adapter_read_clock(adapter, &mcl->addr, which, 1000,
+						btclock, btaccuracy) < 0)
 		return FALSE;
 
-	ret = btd_adapter_read_clock(adapter, handle, which, 1000, btclock,
-								btaccuracy);
-
-	return ret < 0 ? FALSE : TRUE;
+	return TRUE;
 }
 
 static gboolean read_btclock_retry(struct mcap_mcl *mcl, uint32_t *btclock,
