@@ -93,7 +93,7 @@ static void pincode_cb(struct agent *agent, DBusError *derr,
 	bdaddr_t dba;
 	int err;
 
-	device_get_address(device, &dba);
+	device_get_address(device, &dba, NULL);
 
 	if (derr) {
 		err = btd_adapter_pincode_reply(adapter, &dba, NULL, 0);
@@ -138,10 +138,11 @@ static int confirm_reply(struct btd_adapter *adapter,
 				struct btd_device *device, gboolean success)
 {
 	bdaddr_t bdaddr;
+	addr_type_t type;
 
-	device_get_address(device, &bdaddr);
+	device_get_address(device, &bdaddr, &type);
 
-	return btd_adapter_confirm_reply(adapter, &bdaddr, success);
+	return btd_adapter_confirm_reply(adapter, &bdaddr, type, success);
 }
 
 static void confirm_cb(struct agent *agent, DBusError *err, void *user_data)
@@ -159,13 +160,14 @@ static void passkey_cb(struct agent *agent, DBusError *err, uint32_t passkey,
 	struct btd_device *device = user_data;
 	struct btd_adapter *adapter = device_get_adapter(device);
 	bdaddr_t bdaddr;
+	addr_type_t type;
 
-	device_get_address(device, &bdaddr);
+	device_get_address(device, &bdaddr, &type);
 
 	if (err)
 		passkey = INVALID_PASSKEY;
 
-	btd_adapter_passkey_reply(adapter, &bdaddr, passkey);
+	btd_adapter_passkey_reply(adapter, &bdaddr, type, passkey);
 }
 
 int btd_event_user_confirm(bdaddr_t *sba, bdaddr_t *dba, uint32_t passkey)
@@ -204,24 +206,6 @@ int btd_event_user_notify(bdaddr_t *sba, bdaddr_t *dba, uint32_t passkey)
 								FALSE, NULL);
 }
 
-void btd_event_bonding_complete(bdaddr_t *local, bdaddr_t *peer,
-							uint8_t status)
-{
-	struct btd_adapter *adapter;
-	struct btd_device *device;
-	gboolean create;
-
-	DBG("status 0x%02x", status);
-
-	create = status ? FALSE : TRUE;
-
-	if (!get_adapter_and_device(local, peer, &adapter, &device, create))
-		return;
-
-	if (device)
-		device_bonding_complete(device, status);
-}
-
 void btd_event_simple_pairing_complete(bdaddr_t *local, bdaddr_t *peer,
 								uint8_t status)
 {
@@ -264,7 +248,7 @@ static void update_lastused(bdaddr_t *sba, bdaddr_t *dba)
 	write_lastused_info(sba, dba, tm);
 }
 
-void btd_event_device_found(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
+void btd_event_device_found(bdaddr_t *local, bdaddr_t *peer, addr_type_t type,
 					int8_t rssi, uint8_t confirm_name,
 					uint8_t *data, uint8_t data_len)
 {
@@ -277,12 +261,11 @@ void btd_event_device_found(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 	}
 
 	update_lastseen(local, peer);
-	write_remote_class(local, peer, class);
 
 	if (data)
-		write_remote_eir(local, peer, data);
+		write_remote_eir(local, peer, data, data_len);
 
-	adapter_update_found_devices(adapter, peer, class, rssi,
+	adapter_update_found_devices(adapter, peer, type, rssi,
 						confirm_name, data, data_len);
 }
 
@@ -290,7 +273,7 @@ void btd_event_set_legacy_pairing(bdaddr_t *local, bdaddr_t *peer,
 							gboolean legacy)
 {
 	struct btd_adapter *adapter;
-	struct remote_dev_info *dev, match;
+	struct remote_dev_info *dev;
 
 	adapter = manager_find_adapter(local);
 	if (!adapter) {
@@ -298,11 +281,7 @@ void btd_event_set_legacy_pairing(bdaddr_t *local, bdaddr_t *peer,
 		return;
 	}
 
-	memset(&match, 0, sizeof(struct remote_dev_info));
-	bacpy(&match.bdaddr, peer);
-	match.name_status = NAME_ANY;
-
-	dev = adapter_search_found_devices(adapter, &match);
+	dev = adapter_search_found_devices(adapter, peer);
 	if (dev)
 		dev->legacy = legacy;
 }
@@ -329,44 +308,33 @@ void btd_event_remote_class(bdaddr_t *local, bdaddr_t *peer, uint32_t class)
 	device_set_class(device, class);
 }
 
-void btd_event_remote_name(bdaddr_t *local, bdaddr_t *peer, uint8_t status,
-				char *name)
+void btd_event_remote_name(bdaddr_t *local, bdaddr_t *peer, char *name)
 {
 	struct btd_adapter *adapter;
-	char srcaddr[18], dstaddr[18];
+	char srcaddr[18];
 	struct btd_device *device;
-	struct remote_dev_info match, *dev_info;
+	struct remote_dev_info *dev_info;
 
-	if (status == 0) {
-		if (!g_utf8_validate(name, -1, NULL)) {
-			int i;
+	if (!g_utf8_validate(name, -1, NULL)) {
+		int i;
 
-			/* Assume ASCII, and replace all non-ASCII with
-			 * spaces */
-			for (i = 0; name[i] != '\0'; i++) {
-				if (!isascii(name[i]))
-					name[i] = ' ';
-			}
-			/* Remove leading and trailing whitespace characters */
-			g_strstrip(name);
+		/* Assume ASCII, and replace all non-ASCII with spaces */
+		for (i = 0; name[i] != '\0'; i++) {
+			if (!isascii(name[i]))
+				name[i] = ' ';
 		}
-
-		write_device_name(local, peer, name);
+		/* Remove leading and trailing whitespace characters */
+		g_strstrip(name);
 	}
 
+	write_device_name(local, peer, name);
+
 	if (!get_adapter_and_device(local, peer, &adapter, &device, FALSE))
 		return;
 
 	ba2str(local, srcaddr);
-	ba2str(peer, dstaddr);
 
-	if (status != 0)
-		goto proceed;
-
-	bacpy(&match.bdaddr, peer);
-	match.name_status = NAME_ANY;
-
-	dev_info = adapter_search_found_devices(adapter, &match);
+	dev_info = adapter_search_found_devices(adapter, peer);
 	if (dev_info) {
 		g_free(dev_info->name);
 		dev_info->name = g_strdup(name);
@@ -375,16 +343,56 @@ void btd_event_remote_name(bdaddr_t *local, bdaddr_t *peer, uint8_t status,
 
 	if (device)
 		device_set_name(device, name);
+}
 
-proceed:
-	/* remove from remote name request list */
-	adapter_remove_found_device(adapter, peer);
+static char *buf2str(uint8_t *data, int datalen)
+{
+	char *buf;
+	int i;
 
-	/* check if there is more devices to request names */
-	if (adapter_resolve_names(adapter) == 0)
-		return;
+	buf = g_try_new0(char, (datalen * 2) + 1);
+	if (buf == NULL)
+		return NULL;
 
-	adapter_set_state(adapter, STATE_IDLE);
+	for (i = 0; i < datalen; i++)
+		sprintf(buf + (i * 2), "%2.2x", data[i]);
+
+	return buf;
+}
+
+static int store_longtermkey(bdaddr_t *local, bdaddr_t *peer,
+				addr_type_t addr_type, 	unsigned char *key,
+				uint8_t master, uint8_t authenticated,
+				uint8_t enc_size, uint16_t ediv, uint8_t rand[8])
+{
+	GString *newkey;
+	char *val, *str;
+	int err;
+
+	val = buf2str(key, 16);
+	if (val == NULL)
+		return -ENOMEM;
+
+	newkey = g_string_new(val);
+	g_free(val);
+
+	g_string_append_printf(newkey, " %d %d %d %d %d ", addr_type,
+					authenticated, master, enc_size, ediv);
+
+	str = buf2str(rand, 8);
+	if (str == NULL) {
+		g_string_free(newkey, TRUE);
+		return -ENOMEM;
+	}
+
+	newkey = g_string_append(newkey, str);
+	g_free(str);
+
+	err = write_longtermkeys(local, peer, newkey->str);
+
+	g_string_free(newkey, TRUE);
+
+	return err;
 }
 
 int btd_event_link_key_notify(bdaddr_t *local, bdaddr_t *peer,
@@ -412,7 +420,32 @@ int btd_event_link_key_notify(bdaddr_t *local, bdaddr_t *peer,
 	return ret;
 }
 
-void btd_event_conn_complete(bdaddr_t *local, bdaddr_t *peer)
+int btd_event_ltk_notify(bdaddr_t *local, bdaddr_t *peer, addr_type_t addr_type,
+					uint8_t *key, uint8_t master,
+					uint8_t authenticated, uint8_t enc_size,
+					uint16_t ediv, 	uint8_t rand[8])
+{
+	struct btd_adapter *adapter;
+	struct btd_device *device;
+	int ret;
+
+	if (!get_adapter_and_device(local, peer, &adapter, &device, TRUE))
+		return -ENODEV;
+
+	ret = store_longtermkey(local, peer, addr_type, key, master,
+					authenticated, enc_size, ediv, rand);
+	if (ret == 0) {
+		device_set_bonded(device, TRUE);
+
+		if (device_is_temporary(device))
+			device_set_temporary(device, FALSE);
+	}
+
+	return ret;
+}
+
+void btd_event_conn_complete(bdaddr_t *local, bdaddr_t *peer, addr_type_t type,
+						char *name, uint8_t *dev_class)
 {
 	struct btd_adapter *adapter;
 	struct btd_device *device;
@@ -422,7 +455,12 @@ void btd_event_conn_complete(bdaddr_t *local, bdaddr_t *peer)
 
 	update_lastused(local, peer);
 
+	device_set_addr_type(device, type);
+
 	adapter_add_connection(adapter, device);
+
+	if (name != NULL)
+		btd_event_remote_name(local, peer, name);
 }
 
 void btd_event_conn_failed(bdaddr_t *local, bdaddr_t *peer, uint8_t status)
@@ -472,7 +510,8 @@ void btd_event_device_blocked(bdaddr_t *local, bdaddr_t *peer)
 	if (!get_adapter_and_device(local, peer, &adapter, &device, FALSE))
 		return;
 
-	device_block(conn, device, TRUE);
+	if (device)
+		device_block(conn, device, TRUE);
 }
 
 void btd_event_device_unblocked(bdaddr_t *local, bdaddr_t *peer)
@@ -485,7 +524,25 @@ void btd_event_device_unblocked(bdaddr_t *local, bdaddr_t *peer)
 	if (!get_adapter_and_device(local, peer, &adapter, &device, FALSE))
 		return;
 
-	device_unblock(conn, device, FALSE, TRUE);
+	if (device)
+		device_unblock(conn, device, FALSE, TRUE);
+}
+
+void btd_event_device_unpaired(bdaddr_t *local, bdaddr_t *peer)
+{
+	struct btd_adapter *adapter;
+	struct btd_device *device;
+	DBusConnection *conn = get_dbus_connection();
+
+	if (!get_adapter_and_device(local, peer, &adapter, &device, FALSE))
+		return;
+
+	device_set_temporary(device, TRUE);
+
+	if (device_is_connected(device))
+		device_request_disconnect(device, NULL);
+	else
+		adapter_remove_device(conn, adapter, device, TRUE);
 }
 
 /* Section reserved to device HCI callbacks */
