@@ -35,8 +35,6 @@
 #include <time.h>
 #include <sys/file.h>
 #include <sys/stat.h>
-#include <sys/param.h>
-#include <sys/socket.h>
 
 #include <glib.h>
 
@@ -45,8 +43,7 @@
 #include <bluetooth/sdp_lib.h>
 
 #include "textfile.h"
-#include "adapter.h"
-#include "device.h"
+#include "glib-compat.h"
 #include "glib-helper.h"
 #include "storage.h"
 
@@ -241,8 +238,8 @@ int read_local_name(bdaddr_t *bdaddr, char *name)
 		return -ENOENT;
 
 	len = strlen(str);
-	if (len > 248)
-		str[248] = '\0';
+	if (len > HCI_MAX_NAME_LENGTH)
+		str[HCI_MAX_NAME_LENGTH] = '\0';
 	strcpy(name, str);
 
 	free(str);
@@ -323,11 +320,11 @@ int read_remote_class(bdaddr_t *local, bdaddr_t *peer, uint32_t *class)
 
 int write_device_name(bdaddr_t *local, bdaddr_t *peer, char *name)
 {
-	char filename[PATH_MAX + 1], addr[18], str[249];
+	char filename[PATH_MAX + 1], addr[18], str[HCI_MAX_NAME_LENGTH + 1];
 	int i;
 
 	memset(str, 0, sizeof(str));
-	for (i = 0; i < 248 && name[i]; i++)
+	for (i = 0; i < HCI_MAX_NAME_LENGTH && name[i]; i++)
 		if ((unsigned char) name[i] < 32 || name[i] == 127)
 			str[i] = '.';
 		else
@@ -353,8 +350,8 @@ int read_device_name(const char *src, const char *dst, char *name)
 		return -ENOENT;
 
 	len = strlen(str);
-	if (len > 248)
-		str[248] = '\0';
+	if (len > HCI_MAX_NAME_LENGTH)
+		str[HCI_MAX_NAME_LENGTH] = '\0';
 	strcpy(name, str);
 
 	free(str);
@@ -362,13 +359,14 @@ int read_device_name(const char *src, const char *dst, char *name)
 	return 0;
 }
 
-int write_remote_eir(bdaddr_t *local, bdaddr_t *peer, uint8_t *data)
+int write_remote_eir(bdaddr_t *local, bdaddr_t *peer, uint8_t *data,
+							uint8_t data_len)
 {
 	char filename[PATH_MAX + 1], addr[18], str[481];
 	int i;
 
 	memset(str, 0, sizeof(str));
-	for (i = 0; i < HCI_MAX_EIR_LENGTH; i++)
+	for (i = 0; i < data_len; i++)
 		sprintf(str + (i * 2), "%2.2X", data[i]);
 
 	create_filename(filename, PATH_MAX, local, "eir");
@@ -732,35 +730,6 @@ gboolean read_trust(const bdaddr_t *local, const char *addr, const char *service
 	return ret;
 }
 
-struct trust_list {
-	GSList *trusts;
-	const char *service;
-};
-
-static void append_trust(char *key, char *value, void *data)
-{
-	struct trust_list *list = data;
-
-	if (strstr(value, list->service))
-		list->trusts = g_slist_append(list->trusts, g_strdup(key));
-}
-
-GSList *list_trusts(bdaddr_t *local, const char *service)
-{
-	char filename[PATH_MAX + 1];
-	struct trust_list list;
-
-	create_filename(filename, PATH_MAX, local, "trusts");
-
-	list.trusts = NULL;
-	list.service = service;
-
-	if (textfile_foreach(filename, append_trust, &list) < 0)
-		return NULL;
-
-	return list.trusts;
-}
-
 int write_device_profiles(bdaddr_t *src, bdaddr_t *dst, const char *profiles)
 {
 	char filename[PATH_MAX + 1], addr[18];
@@ -1110,8 +1079,6 @@ int read_device_pairable(bdaddr_t *bdaddr, gboolean *mode)
 
 	create_filename(filename, PATH_MAX, bdaddr, "config");
 
-	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-
 	str = textfile_get(filename, "pairable");
 	if (!str)
 		return -ENOENT;
@@ -1174,62 +1141,54 @@ int write_device_services(const bdaddr_t *sba, const bdaddr_t *dba,
 static void filter_keys(char *key, char *value, void *data)
 {
 	struct match *match = data;
-	const char *address = match->pattern;
 
-	/* Each key contains: MAC#handle*/
-	if (strncasecmp(key, address, 17) == 0)
+	if (strncasecmp(key, match->pattern, strlen(match->pattern)) == 0)
 		match->keys = g_slist_append(match->keys, g_strdup(key));
 }
 
+static void delete_by_pattern(const char *filename, char *pattern)
+{
+	struct match match;
+	GSList *l;
+	int err;
+
+	memset(&match, 0, sizeof(match));
+	match.pattern = pattern;
+
+	err = textfile_foreach(filename, filter_keys, &match);
+	if (err < 0)
+		goto done;
+
+	for (l = match.keys; l; l = l->next) {
+		const char *key = l->data;
+		textfile_del(filename, key);
+	}
+
+done:
+	g_slist_free_full(match.keys, g_free);
+}
+
 int delete_device_service(const bdaddr_t *sba, const bdaddr_t *dba)
 {
-	GSList *l;
-	struct match match;
 	char filename[PATH_MAX + 1], address[18];
-	int err;
-
-	create_filename(filename, PATH_MAX, sba, "primary");
 
 	memset(address, 0, sizeof(address));
 	ba2str(dba, address);
 
-	err = textfile_del(filename, address);
-	if (err < 0)
-		return err;
-
 	/* Deleting all characteristics of a given address */
-	memset(&match, 0, sizeof(match));
-	match.pattern = address;
-
 	create_filename(filename, PATH_MAX, sba, "characteristic");
-	err = textfile_foreach(filename, filter_keys, &match);
-	if (err < 0)
-		return err;
-
-	for (l = match.keys; l; l = l->next) {
-		const char *key = l->data;
-		textfile_del(filename, key);
-	}
-
-	g_slist_free_full(match.keys, g_free);
+	delete_by_pattern(filename, address);
 
 	/* Deleting all attributes values of a given address */
-	memset(&match, 0, sizeof(match));
-	match.pattern = address;
-
 	create_filename(filename, PATH_MAX, sba, "attributes");
-	err = textfile_foreach(filename, filter_keys, &match);
-	if (err < 0)
-		return err;
+	delete_by_pattern(filename, address);
 
-	for (l = match.keys; l; l = l->next) {
-		const char *key = l->data;
-		textfile_del(filename, key);
-	}
+	/* Deleting all CCC values of a given address */
+	create_filename(filename, PATH_MAX, sba, "ccc");
+	delete_by_pattern(filename, address);
 
-	g_slist_free_full(match.keys, g_free);
-
-	return 0;
+	create_filename(filename, PATH_MAX, sba, "primary");
+	return textfile_del(filename, address);
 }
 
 char *read_device_services(const bdaddr_t *sba, const bdaddr_t *dba)
@@ -1238,8 +1197,6 @@ char *read_device_services(const bdaddr_t *sba, const bdaddr_t *dba)
 
 	create_filename(filename, PATH_MAX, sba, "primary");
 
-	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-
 	ba2str(dba, addr);
 
 	return textfile_caseget(filename, addr);
@@ -1268,8 +1225,6 @@ char *read_device_characteristics(const bdaddr_t *sba, const bdaddr_t *dba,
 
 	create_filename(filename, PATH_MAX, sba, "characteristic");
 
-	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-
 	ba2str(dba, addr);
 
 	snprintf(key, sizeof(key), "%17s#%04X", addr, handle);
@@ -1299,45 +1254,92 @@ int read_device_attributes(const bdaddr_t *sba, textfile_cb func, void *data)
 
 	create_filename(filename, PATH_MAX, sba, "attributes");
 
-	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-
 	return textfile_foreach(filename, func, data);
 }
 
-int write_device_type(const bdaddr_t *sba, const bdaddr_t *dba,
-						device_type_t type)
+int read_device_ccc(bdaddr_t *local, bdaddr_t *peer, uint16_t handle,
+							uint16_t *value)
 {
-	char filename[PATH_MAX + 1], addr[18], chars[3];
+	char filename[PATH_MAX + 1], addr[18], key[23];
+	char *str;
+	unsigned int config;
+	int err = 0;
 
-	create_filename(filename, PATH_MAX, sba, "types");
+	create_filename(filename, PATH_MAX, local, "ccc");
+
+	ba2str(peer, addr);
+	snprintf(key, sizeof(key), "%17s#%04X", addr, handle);
+
+	str = textfile_caseget(filename, key);
+	if (str == NULL)
+		return -ENOENT;
+
+	if (sscanf(str, "%04X", &config) != 1)
+		err = -ENOENT;
+	else
+		*value = config;
+
+	free(str);
+
+	return err;
+}
+
+int write_device_ccc(bdaddr_t *local, bdaddr_t *peer, uint16_t handle,
+							uint16_t value)
+{
+	char filename[PATH_MAX + 1], addr[18], key[23], config[5];
+
+	create_filename(filename, PATH_MAX, local, "ccc");
 
 	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 
-	ba2str(dba, addr);
+	ba2str(peer, addr);
 
-	snprintf(chars, sizeof(chars), "%2.2X", type);
+	snprintf(key, sizeof(key), "%17s#%04X", addr, handle);
+	snprintf(config, sizeof(config), "%04X", value);
 
-	return textfile_put(filename, addr, chars);
+	return textfile_put(filename, key, config);
 }
 
-device_type_t read_device_type(const bdaddr_t *sba, const bdaddr_t *dba)
+void delete_device_ccc(bdaddr_t *local, bdaddr_t *peer)
 {
-	char filename[PATH_MAX + 1], addr[18], *chars;
-	device_type_t type;
+	char filename[PATH_MAX + 1], addr[18];
 
-	create_filename(filename, PATH_MAX, sba, "types");
+	ba2str(peer, addr);
+
+	/* Deleting all CCC values of a given address */
+	create_filename(filename, PATH_MAX, local, "ccc");
+	delete_by_pattern(filename, addr);
+}
+
+int write_longtermkeys(bdaddr_t *local, bdaddr_t *peer, const char *key)
+{
+	char filename[PATH_MAX + 1], addr[18];
+
+	if (!key)
+		return -EINVAL;
+
+	create_filename(filename, PATH_MAX, local, "longtermkeys");
 
 	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 
-	ba2str(dba, addr);
+	ba2str(peer, addr);
+	return textfile_put(filename, addr, key);
+}
 
-	chars = textfile_caseget(filename, addr);
-	if (chars == NULL)
-		return DEVICE_TYPE_UNKNOWN;
+gboolean has_longtermkeys(bdaddr_t *local, bdaddr_t *peer)
+{
+	char filename[PATH_MAX + 1], addr[18], *str;
 
-	type = strtol(chars, NULL, 16);
+	create_filename(filename, PATH_MAX, local, "longtermkeys");
 
-	free(chars);
+	ba2str(peer, addr);
 
-	return type;
+	str = textfile_caseget(filename, addr);
+	if (str) {
+		free(str);
+		return TRUE;
+	}
+
+	return FALSE;
 }
