@@ -148,8 +148,8 @@ static int write_cmd(int fd, uint8_t *buffer, int len)
 		return err;
 
 	err = read_ps_event(event, HCI_PS_CMD_OCF);
-	if (event)
-		free(event);
+
+	free(event);
 
 	return err;
 }
@@ -493,8 +493,7 @@ static int set_patch_ram(int dev, char *patch_loc, int len)
 
 	err = read_ps_event(event, HCI_PS_CMD_OCF);
 
-	if (event)
-		free(event);
+	free(event);
 
 	return err;
 }
@@ -623,8 +622,7 @@ static int get_ath3k_crc(int dev)
 	if (read_ps_event(event, HCI_PS_CMD_OCF) >= 0)
 		err = -EILSEQ;
 
-	if (!event)
-		free(event);
+	free(event);
 
 	return err;
 }
@@ -667,8 +665,7 @@ static int get_device_type(int dev, uint32_t *code)
 	*code = reg;
 
 cleanup:
-	if (event)
-		free(event);
+	free(event);
 
 	return err;
 }
@@ -709,8 +706,7 @@ static int read_ath3k_version(int pConfig, uint32_t *rom_version,
 	*build_version = status;
 
 cleanup:
-	if (event)
-		free(event);
+	free(event);
 
 	return err;
 }
@@ -768,8 +764,7 @@ static int write_bdaddr(int pConfig, char *bdaddr)
 
 	err = read_ps_event(event, HCI_PS_CMD_OCF);
 
-	if (event)
-		free(event);
+	free(event);
 
 	return err;
 }
